"""
 Copyright (c) 2019 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
import argparse
import psutil
import signal
import subprocess
import sys
import threading
import time


# import webbrowser


class S3P(threading.Thread):
    """
    This class starts the Banyan server to support Scratch 3 OneGPIO
    for the Picoboard

    It will start the backplane, picoboard gateway and websocket gateway.
    """

    def __init__(self, com_port=None, log=None):
        """
        :param com_port: Manually select serial com port
        :param log: log unhandled exceptions.
        """

        self.com_port = com_port
        self.log = log

        self.backplane_exists = False

        # pids of processes
        self.proc_bp = None
        self.proc_awg = None
        self.proc_hwg = None

        # start backplane
        self.start_backplane()

        # start the websocket gateway
        self.start_wsgw()

        # start picoboard gateway
        self.start_pbgw()

        # webbrowser.open('https://mryslab.github.io/s3onegpio/', new=1)

        # start the thread to check if processes are still alive
        threading.Thread.__init__(self)
        self.daemon = True
        self.stop_event = threading.Event()
        self.stop_event.clear()
        self.start()

        while True:
            if self.stop_event.is_set():
                print('Exiting. Error detected - is your Picoboard plugged in?')
                sys.exit(0)
            try:
                time.sleep(.01)
            except KeyboardInterrupt:
                sys.exit(0)

    def run(self):
        """
        The thread code to monitor if all processes are still alive.
        """
        while not self.stop_event.is_set():
            bp = ws = pb = None
            try:
                bp = [p.info for p in psutil.process_iter(attrs=['pid', 'name', 'status'])
                      if 'backplane' in p.info['name']]
            except psutil.NoSuchProcess:
                pass
            try:
                ws = [p.info for p in psutil.process_iter(attrs=['pid', 'name', 'status'])
                      if 'wsgw' in p.info['name']]
            except psutil.NoSuchProcess:
                pass
            try:
                pb = [p.info for p in psutil.process_iter(attrs=['pid', 'name', 'status'])
                      if 'pbgw' in p.info['name']]
            except psutil.NoSuchProcess:
                pass

            if not bp:
                self.start_backplane()
                print('start backplane')

            try:
                if bp[0]['status'] == 'zombie':
                    self.killall(bp, ws, pb)
            except IndexError:
                pass

            z = [p.info for p in psutil.process_iter(attrs=['pid', 'name', 'status'])
                 if 'wsgw' in p.info['name']]

            if not z:
                self.killall(bp, ws, pb)
            try:
                if z[0]['status'] == 'zombie':
                    self.killall(bp, ws, pb)
            except IndexError:
                pass

            z = [p.info for p in psutil.process_iter(attrs=['pid', 'name', 'status'])
                 if 'pbgw' in p.info['name']]

            if not z:
                self.killall(bp, ws, pb)

            try:
                if z[0]['status'] == 'zombie':
                    self.killall(bp, ws, pb)
            except IndexError:
                pass

            time.sleep(.1)

    def killall(self, b, w, p):
        """
        Kill all running or zombie processes
        :param b: backplane
        :param w: websocket gateway
        :param p: picoboard gateway
        """

        self.stop_event.set()
        # check for missing processes
        if b:
            try:
                p = psutil.Process(self.proc_bp)
            except psutil.NoSuchProcess:
                pass
            else:
                p.kill()
        if w:
            try:
                w = psutil.Process(self.proc_awg)
            except psutil.NoSuchProcess:
                pass
            else:
                p.kill()
        if p:
            try:
                p = psutil.Process(self.proc_hwg)
            except psutil.NoSuchProcess:
                pass
            else:
                p.kill()

    def start_backplane(self):
        """
        Start the backplane
        """
        for pid in psutil.pids():
            p = psutil.Process(pid)
            if p.name() == "backplane":
                print("Backplane already started.          PID = " + str(p.pid))
                self.backplane_exists = True
                self.proc_bp = p.pid
            else:
                continue

        # if backplane is not already running, start a new instance
        if not self.backplane_exists:
            if sys.platform.startswith('win32'):
                self.proc_bp = subprocess.Popen('backplane',
                                                creationflags=subprocess.CREATE_NO_WINDOW).pid

            else:
                self.proc_bp = subprocess.Popen('backplane',
                                                stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                                stdout=subprocess.PIPE).pid
            self.backplane_exists = True
            print('Backplane is started...')

    def start_wsgw(self):
        """
        Start the websocket gateway
        """
        if sys.platform.startswith('win32'):
            wsgw_start = ['wsgw', '-i', '9004']
        else:
            wsgw_start = ['wsgw', '-i', '9004']

        if self.log:
            wsgw_start.append('-l')
            wsgw_start.append('True')

        if sys.platform.startswith('win32'):
            self.proc_awg = subprocess.Popen(wsgw_start,
                                             creationflags=subprocess.CREATE_NO_WINDOW).pid

        else:
            self.proc_awg = subprocess.Popen(wsgw_start,
                                             stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                             stdout=subprocess.PIPE).pid

        print('WebSocket Gateway is started...')

    def start_pbgw(self):
        """
        Start the picoboard gateway
        """
        if sys.platform.startswith('win32'):
            pbgw_start = ['pbgw']
        else:
            pbgw_start = ['pbgw']

        if self.log:
            pbgw_start.append('-l')
            pbgw_start.append('True')
        if self.com_port:
            pbgw_start.append('-c')
            pbgw_start.append(self.com_port)

        if sys.platform.startswith('win32'):
            self.proc_hwg = subprocess.Popen(pbgw_start,
                                             creationflags=subprocess.CREATE_NO_WINDOW).pid
        else:
            self.proc_hwg = subprocess.Popen(pbgw_start,
                                             stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                             stdout=subprocess.PIPE).pid

        print('Picoboard Gateway is started...')


def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


def s3px():
    """
    Start the s3p script
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", dest="com_port", default="None",
                        help="Use this COM port instead of auto discovery")
    parser.add_argument("-l", dest="log", default="False",
                        help="Set to True to turn logging on.")

    args = parser.parse_args()

    log = args.log.lower()
    if log == 'false':
        log = False
    else:
        log = True

    if args.com_port == "None":
        com_port = None
    else:
        com_port = args.com_port

    S3P(com_port=com_port, log=log)


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    # replace with name of function you defined above
    s3px()



"""
 Copyright (c) 2019 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import psutil
import signal
import subprocess
import sys
import time


# import webbrowser


class S3R:
    """
    This class starts the Banyan server to support the Scratch 3 OneGPIO Raspberry Pi
    extension for local mode. That means the browser and components are all executing
    on a single RPi.

    It will start the backplane, Raspberry Pi gateway and websocket gateway.
    """

    def __init__(self):

        # backplane status string
        # bp_string = ""
        self.backplane_exists = False
        self.proc_bp = None
        self.proc_agw = None
        self.proc_hwg = None

        print("Only run this script on a Raspberry Pi!")

        # checking running processes.
        # if the backplane is already running, just note that and move on.
        for pid in psutil.pids():
            p = psutil.Process(pid)
            if p.name() == "backplane":
                # new_entry['process_id'] = p
                # bp_string = "Backplane already running.          PID = " + str(p.pid)
                print("Backplane already running.          PID = " + str(p.pid))
                self.backplane_exists = True
            else:
                continue

        # if backplane is not already running, start a new instance
        if not self.backplane_exists:
            self.proc_bp = subprocess.Popen(['backplane'],
                                            stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                            stdout=subprocess.PIPE)
            self.backplane_exists = True
            print('backplane started')
        if sys.platform.startswith('win32'):
            print('This must be run on a Raspberry Pi.')
            sys.exit(0)
        else:
            self.proc_awg = subprocess.Popen(['xterm', '-e', 'wsgw -i 9001'],
                                             stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                             stdout=subprocess.PIPE)
            self.proc_hwg = subprocess.Popen(['xterm', '-e', 'rpigw'],
                                             stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                             stdout=subprocess.PIPE)
        # webbrowser.open('https://mryslab.github.io/s3onegpio/')

        while True:
            try:
                time.sleep(1)
            except KeyboardInterrupt:
                sys.exit(0)


def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


def s3rx():
    S3R()


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    # replace with name of function you defined above
    s3rx()



import serial
from serial.tools import list_ports
import sys
import threading
import time

# class CpeGateway(threading.Thread):
class CpeGateway(threading.Thread):

    def __init__(self):

        self.start_time = None

        threading.Thread.__init__(self)
        self.daemon = True
        self.stop_event = threading.Event()
        try:
            self.cpe_serial = serial.Serial(port="/dev/ttyACM0", baudrate=115200,
                                                  timeout=.01)
        except serial.SerialException:
            print('Connection to Circuit Playground Expressed Failed.')
            sys.exit(0)
        except OSError:
            pass
        self.start()
        time.sleep(1)

        # get version
        cmd = 'v\r'.encode()
        self.cpe_serial.write(cmd)
        self.start_time = time.time()

        cmd = 'acc\r'.encode()
        self.cpe_serial.write(cmd)
        self.start_time = time.time()

        cmd = 'temp\r'.encode()
        self.cpe_serial.write(cmd)
        self.start_time = time.time()

        while True:
           time.sleep(.001)


    def run(self):
        """
        This method continually runs. If an incoming character is available on the serial port
        it is read and placed on the _command_deque
        @return: Never Returns
        """
        while True:
            # we can get an OSError: [Errno9] Bad file descriptor when shutting down
            # just ignore it
            try:
                if self.cpe_serial.inWaiting():
                    c = self.cpe_serial.readline().decode()
                    print(time.time() - self.start_time)

                    print(c)
                else:
                    time.sleep(.1)
            except OSError:
                pass
            # except IOError:
            #     self.stop()
            #     self.close()





CpeGateway()



#!/usr/bin/env python3

"""
 This is the Python Banyan GUI that communicates with
 the Raspberry Pi Banyan Gateway

 Copyright (c) 2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
import argparse
import signal
from subprocess import run
import sys
import time

import pigpio
from python_banyan.gateway_base import GatewayBase

from .sonar import Sonar
from .stepper import StepperMotor

import zmq


# noinspection PyAbstractClass
class RpiGateway(GatewayBase):
    """
    This class implements a Banyan gateway for the Raspberry Pi
    GPIO pins. It implements the Common Unified GPIO Message
    Specification.

    If pipgiod is not currently running, it will start it, and
    no backplane ip address was specified, a local backplane
    will be automatically started. If you specify a backplane
    ip address, you will need to start that backplane manually.
    """

    def __init__(self, *subscriber_list, **kwargs):
        """
        Initialize the class for operation
        :param subscriber_list: A list of subscription topics
        :param kwargs: Command line arguments - see bg4rpi()
                       at the bottom of this file.
        """

        # attempt to instantiate pigpio
        self.pi = pigpio.pi()

        # if pigpiod is not running, try to start it
        #
        if not self.pi.connected:
            print('\nAttempting to start pigpiod...')
            run(['sudo', 'pigpiod'])
            time.sleep(.5)
            self.pi = pigpio.pi()
            if not self.pi.connected:
                print('pigpiod did not start - goodbye.')
                sys.exit()
            else:
                print('pigpiod successfully started!')

        print('pigpiod is running version: ', self.pi.get_pigpio_version())

        # variables to hold instances of sonar and stepper
        self.sonar = None
        self.stepper = None

        # a list of valid pin numbers
        # using a list comprehension to create the list
        self.gpio_pins = [pin for pin in range(2, 28)]

        # i2c handle supplied by pigpio when i2c open is called
        self.i2c_handle = None

        # initialize the parent
        super(RpiGateway, self).__init__(subscriber_list=subscriber_list,
                                         back_plane_ip_address=kwargs['back_plane_ip_address'],
                                         subscriber_port=kwargs['subscriber_port'],
                                         publisher_port=kwargs['publisher_port'],
                                         process_name=kwargs['process_name'],
                                         )

        # start the banyan receive loop
        try:
            self.receive_loop()
        except KeyboardInterrupt:
            self.pi.stop()
            self.clean_up()
            sys.exit(0)

    def init_pins_dictionary(self):
        """
        This method is called by the gateway_base parent class
        when it is initializing.
        """

        # build a status table for the pins
        for x in self.gpio_pins:
            entry = {'mode': None, 'duty': None, 'freq': None, 'value': 0}
            self.pins_dictionary[x] = entry

    def digital_write(self, topic, payload):
        """
        This method performs a digital write
        :param topic: message topic
        :param payload: {"command": "digital_write", "pin": “PIN”, "value": “VALUE”}
        """
        self.pi.write(payload['pin'], payload['value'])

    def disable_digital_reporting(self, topic, payload):
        """
        This method disables digital input reporting for the selected pin.

        :param topic: message topic
        :param payload: {"command": "disable_digital_reporting", "pin": “PIN”, "tag": "TAG"}
        """
        entry = self.pins_dictionary[payload['pin']]
        entry['mode'] = None

    def i2c_read(self, topic, payload):
        """
        This method will perform an i2c read by specifying the i2c
        device address, i2c device register and the number of bytes
        to read.

        Call set_mode_i2c first to establish the pins for i2c operation.

        :param topic: message topic
        :param payload: {"command": "i2c_read", "pin": “PIN”, "tag": "TAG",
                         "addr": “I2C ADDRESS, "register": “I2C REGISTER”,
                         "number_of_bytes": “NUMBER OF BYTES”}
        :return via the i2c_callback method
        """

        # if no i2c handle has been assigned, get one
        if self.i2c_handle is None:
            self.i2c_handle = self.pi.i2c_open(1, payload['addr'], 0)

        # using the handle do the i2c read and retrieve the data
        value = self.pi.i2c_read_i2c_block_data(self.i2c_handle,
                                                payload['register'],
                                                payload['number_of_bytes'])
        # value index 0 is the number of bytes read.
        # the remainder of value is the a byte array of data returned
        # convert the data to a list
        value = list(value[1])

        # format the data for report to be published containing
        # the data received
        report = ', '.join([str(elem) for elem in value])
        payload = {'report': 'i2c_data', 'value': report}
        self.publish_payload(payload, 'from_rpi_gateway')

    def i2c_write(self, topic, payload):
        """
        This method will perform an i2c write for the i2c device with
        the specified i2c device address, i2c register and a list of byte
        to write.

        Call set_mode_i2c first to establish the pins for i2c operation.

        :param topic: message topic
        :param payload: {"command": "i2c_write", "pin": “PIN”, "tag": "TAG",
                         "addr": “I2C ADDRESS, "register": “I2C REGISTER”,
                         "data": [“DATA IN LIST FORM”]}
        """

        if self.i2c_handle is None:
            self.i2c_handle = self.pi.i2c_open(1, payload['addr'], 0)

        data = payload['data']

        self.pi.i2c_write_device(self.i2c_handle, data)

        # give the i2c device some time to process the write request
        time.sleep(.4)

    def set_mode_tone(self, topic, payload):
        pass

    def play_tone(self, topic, payload):
        """
        This method plays a tone on a piezo device connected to the selected
        pin at the frequency and duration requested.
        Frequency is in hz and duration in milliseconds.

        Call set_mode_tone before using this method.
        :param topic: message topic
        :param payload: {"command": "play_tone", "pin": “PIN”, "tag": "TAG",
                         “freq”: ”FREQUENCY”, duration: “DURATION”}
        """
        pin = int(payload['pin'])
        self.pi.set_mode(pin, pigpio.OUTPUT)

        frequency = int(payload['freq'])
        frequency = int((1000 / frequency) * 1000)
        tone = [pigpio.pulse(1 << pin, 0, frequency),
                pigpio.pulse(0, 1 << pin, frequency)]

        self.pi.wave_add_generic(tone)
        wid = self.pi.wave_create()

        if wid >= 0:
            self.pi.wave_send_repeat(wid)
            time.sleep(payload['duration'] / 1000)
            self.pi.wave_tx_stop()
            self.pi.wave_delete(wid)

    def pwm_write(self, topic, payload):
        """
        This method sets the pwm value for the selected pin.
        Call set_mode_pwm before calling this method.
        :param topic: message topic
        :param payload: {“command”: “pwm_write”, "pin": “PIN”,
                         "tag":”TAG”,
                          “value”: “VALUE”}
        """
        self.pi.set_PWM_dutycycle(payload['pin'], payload['value'])

    def servo_position(self, topic, payload):
        """
        This method will set a servo's position in degrees.
        Call set_mode_servo first to activate the pin for
        servo operation.

        :param topic: message topic
        :param payload: {'command': 'servo_position',
                         "pin": “PIN”,'tag': 'servo',
                        “position”: “POSITION”}
        """
        pin = payload['pin']
        pulse_width = (payload['position'] * 10) + 500
        self.pi.set_servo_pulsewidth(pin, pulse_width)

    def set_mode_analog_input(self, topic, payload):
        """
        This method programs a PCF8591 AD/DA for analog input.
        :param topic: message topic
        :param payload: {"command": "set_mode_analog_input",
                         "pin": “PIN”, "tag":”TAG” }
        """

        # pin is used as channel number

        value = None
        i2c_handle = self.pi.i2c_open(1, 72, 0)
        pin = payload['pin']

        self.pi.i2c_write_byte_data(i2c_handle, 64 | (pin & 0x03), 0)
        time.sleep(0.1)
        for i in range(3):
            value = self.pi.i2c_read_byte(i2c_handle)

        self.pi.i2c_close(i2c_handle)

        # publish an analog input report
        payload = {'report': 'analog_input', 'pin': pin,
                   'value': value}
        self.publish_payload(payload, 'from_rpi_gateway')

    def set_mode_digital_input(self, topic, payload):
        """
        This method sets a pin as digital input.
        :param topic: message topic
        :param payload: {"command": "set_mode_digital_input", "pin": “PIN”, "tag":”TAG” }
        """
        pin = payload['pin']
        entry = self.pins_dictionary[pin]
        entry['mode'] = self.DIGITAL_INPUT_MODE

        self.pi.set_glitch_filter(pin, 20000)
        self.pi.set_mode(pin, pigpio.INPUT)
        self.pi.set_pull_up_down(pin, pigpio.PUD_DOWN)

        self.pi.callback(pin, pigpio.EITHER_EDGE, self.input_callback)

    def set_mode_digital_output(self, topic, payload):
        """
        This method sets a pin as a digital output pin.
        :param topic: message topic
        :param payload: {"command": "set_mode_digital_output",
                         "pin": PIN, "tag":”TAG” }
        """
        self.pi.set_mode(payload['pin'], pigpio.OUTPUT)

    def set_mode_pwm(self, topic, payload):
        """
         This method sets a GPIO pin capable of PWM for PWM operation.
         :param topic: message topic
         :param payload: {"command": "set_mode_pwm", "pin": “PIN”, "tag":”TAG” }
         """
        self.pi.set_mode(payload['pin'], pigpio.OUTPUT)

    def set_mode_servo(self, topic, payload):
        """
        This method establishes a GPIO pin for servo operation.
        :param topic: message topic
        :param payload: {"command": "set_mode_servo", "pin": “PIN”, "tag":”TAG” }
        """
        pass

    def set_mode_sonar(self, topic, payload):
        """
        This method sets the trigger and echo pins for sonar operation.
        :param topic: message topic
        :param payload: {"command": "set_mode_sonar", "trigger_pin": “PIN”, "tag":”TAG”
                         "echo_pin": “PIN”"tag":”TAG” }
        """
        trigger = payload['trigger_pin']
        echo = payload['echo_pin']
        self.sonar = Sonar(self.pi, trigger, echo)
        self.receive_loop_idle_addition = self.read_sonar

    def read_sonar(self):
        """
        Read the sonar device and convert value to
        centimeters. The value is then published as a report.
        """
        sonar_time = self.sonar.read()
        distance = sonar_time / 29 / 2
        distance = round(distance, 2)
        payload = {'report': 'sonar_data', 'value': distance}
        self.publish_payload(payload, 'from_rpi_gateway')

    def set_mode_stepper(self, topic, payload):
        """
        This method establishes either 2 or 4 GPIO pins to be used in stepper
        motor operation.
        :param topic:
        :param payload:{"command": "set_mode_stepper", "pins": [“PINS”],
                        "steps_per_revolution": “NUMBER OF STEPS”}
        """
        self.stepper = StepperMotor(self.pi, payload['pins'][0],
                                    payload['pins'][1],
                                    payload['pins'][2],
                                    payload['pins'][3])

    def stepper_write(self, topic, payload):
        """
        Move a stepper motor for the specified number of steps.
        :param topic:
        :param payload: {"command": "stepper_write", "motor_speed": “SPEED”,
                         "number_of_steps":”NUMBER OF STEPS” }
        """
        if not self.stepper:
            raise RuntimeError('Stepper was not initialized')

        number_of_steps = payload['number_of_steps']
        if number_of_steps >= 0:
            for i in range(number_of_steps):
                self.stepper.do_clockwise_step()
        else:
            for i in range(abs(number_of_steps)):
                self.stepper.do_counterclockwise_step()

    def input_callback(self, pin, level, tick):
        """
        This method is called by pigpio when it detects a change for
        a digital input pin. A report is published reflecting
        the change of pin state for the pin.
        :param pin:
        :param level:
        :param tick:
        :return:
        """
        # payload = {'report': 'digital_input_change', 'pin': str(pin), 'level': str(level)}
        entry = self.pins_dictionary[pin]
        if entry['mode'] == self.DIGITAL_INPUT_MODE:
            payload = {'report': 'digital_input', 'pin': pin,
                       'value': level, 'timestamp': time.time()}
            self.publish_payload(payload, 'from_rpi_gateway')


def rpi_gateway():
    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="back_plane_ip_address", default="None",
                        help="None or IP address used by Back Plane")
    parser.add_argument("-m", dest="subscriber_list", default="to_rpi_gateway", nargs='+',
                        help="Banyan topics space delimited: topic1 topic2 topic3")
    parser.add_argument("-n", dest="process_name", default="RaspberryPiGateway",
                        help="Set process name in banner")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")
    parser.add_argument("-t", dest="loop_time", default=".1",
                        help="Event Loop Timer in seconds")

    args = parser.parse_args()
    if args.back_plane_ip_address == 'None':
        args.back_plane_ip_address = None
    kw_options = {
        'back_plane_ip_address': args.back_plane_ip_address,
        'publisher_port': args.publisher_port,
        'subscriber_port': args.subscriber_port,
        'process_name': args.process_name,
        'loop_time': float(args.loop_time)}

    try:
        RpiGateway(args.subscriber_list, **kw_options)
    except KeyboardInterrupt:
        sys.exit()


def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)


if __name__ == '__main__':
    # replace with name of function you defined above
    rpi_gateway()



"""
 Copyright (c) 2019 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
import serial
from serial.tools import list_ports
import sys
import threading
import time

class PicoBoardReader(threading.Thread):

    """
    Picoboard data reader
    """

    def __init__(self, com_port=None):
        self.baud_rate = 38400

        self.channels = {0: "D", 1: "C",
                           2: "B", 3: "btn", 4: "A",
                           5: "lt", 6: "snd", 7: "slide", 15: "id"}

        self.data_packet = None
        if com_port:
            self.picoboard = serial.Serial(com_port, self.baud_rate,
                                     timeout=1, writeTimeout=0)
        else:
            self.find_the_picoboard()
            print('picoboard found on:', self.picoboard.port)

        threading.Thread.__init__(self)
        self.daemon = True
        self.stop_event = threading.Event()
        self.start()
        time.sleep(.2)
        self.count = 0
        x= b'\x01'

        while True:
            try:
                time.sleep(.001)
                self.picoboard.write(x)
            except KeyboardInterrupt:
                self.picoboard.close()
                sys.exit(0)

    def find_the_picoboard(self):
        # go through the ports looking for an active board
        serial_ports = []

        the_ports_list = list_ports.comports()
        for port in the_ports_list:
            if port.pid is None:
                continue
            else:
                print('Looking for picoboard on: ', port.device)
                self.picoboard = serial.Serial(port.device, self.baud_rate,
                                               timeout=1, writeTimeout=0)
                num_bytes = None
                for send in range(10):
                    self.picoboard.write(b'\x01')
                    time.sleep(.2)
                    num_bytes = self.picoboard.inWaiting()
                    if num_bytes == 18:
                        self.picoboard.reset_input_buffer()
                        self.picoboard.reset_output_buffer()

                        return
                    else:
                        continue
            continue
        print('Could not find a picoboard')
        sys.exit(0)


    def analog_scaling(self, value):
        """
        scale the normal analog input range of 0-1023 to 0-100
        :param value:
        :return:
        """
        assert 0 <= value <= 1023, 'analog input must be in range of 0-1023'
        return int(round ((100 * value) / 1023))

    def light_scaling(self, value):
        if value < 25:
            return 100 - value
        else:
            return int(round((1023 - value) * (75 / 998)))

    def sound_scaling(self, value):
        value = max(0, value - 18)
        if (value < 50):
            return int(value / 2)
        else:
            return int(25 + min(75, round((value - 50) * (75 / 580))))

    def run(self):
        """
        This method continually runs. If an incoming character is available on the serial port
        it is read and placed on the _command_deque
        @return: Never Returns
        """
        # time.sleep(.5)
        # while not self.is_stopped():
        while True:
            # we can get an OSError: [Errno9] Bad file descriptor when shutting down
            # just ignore it
            # try:
            self.data_packet = None
            if self.picoboard.inWaiting():
                self.count += 1
                self.data_packet = self.picoboard.read(18)

                sresult = ''
                for i in range(9):
                    cooked = 0
                    sb_channel = self.channels[(int(self.data_packet[2 * i]) - 128) >> 3]
                    sb_value = ((int(self.data_packet[2 * i]) & 7) << 7) + int(self.data_packet[2 * i + 1])
                    if i in [0, 1, 2, 4, 7]:
                        # scale for standard analog:
                        cooked = self.analog_scaling(sb_value)
                        sresult = sresult + sb_channel + '=' + str(cooked) + ' '
                    elif i == 5: # light
                        cooked = self.light_scaling(sb_value)
                        sresult = sresult + sb_channel + '=' + str(cooked)+ ' '

                    elif i == 6: # sound
                        cooked = self.sound_scaling(sb_value)
                        sresult = sresult + sb_channel + '=' + str(cooked)+ ' '

                    elif i == 3: # invert digital input
                        cooked = not sb_value
                        sresult = sresult + sb_channel + '=' + str(cooked)+ ' '

                    elif i == 8:
                        cooked = sb_value
                        sresult = sresult + sb_channel + '=' + str(cooked)+ ' '

                print('\r' + sresult, end=' ' )

            else:
                try:
                    time.sleep(.001)
                except KeyboardInterrupt:
                    sys.exit(0)

PicoBoardReader()



import serial
import sys
import time

try:
    ser = serial.Serial(port='/dev/ttyACM0', baudrate=115200,
                                                  timeout=.01)
except serial.SerialException:
    print('Connection to Circuit Playground Expressed Failed.')
    sys.exit(0)

while True:

    data = 'p\r'
    data = data.encode()

    # while not ser.inWaiting():
    #     time.sleep(.01)
    # data = ser.readline().decode()
    # print('a', data)
    # cmd = 'v\n'.encode()
    # ser.write(cmd)

    ser.write(data)
    time.sleep(1)
    data = 'n\r'
    data = data.encode()
    ser.write(data)
    time.sleep(1)



# modified from https://github.com/stripcode/pigpio-stepper-motor

from collections import deque
from time import sleep
import pigpio

fullStepSequence = (
    (1, 0, 0, 0),
    (0, 1, 0, 0),
    (0, 0, 1, 0),
    (0, 0, 0, 1)
)

halfStepSequence = (
    (1, 0, 0, 0),
    (1, 1, 0, 0),
    (0, 1, 0, 0),
    (0, 1, 1, 0),
    (0, 0, 1, 0),
    (0, 0, 1, 1),
    (0, 0, 0, 1),
    (1, 0, 0, 1)
)


class StepperMotor:

    def __init__(self, pi, pin1, pin2, pin3, pin4, sequence=fullStepSequence, delay_after_step=0.0025):
        if not isinstance(pi, pigpio.pi):
            raise TypeError("Pigpio instance not found.")
        self.pi = pi

        self.pi.set_mode(pin1, pigpio.OUTPUT)
        self.pi.set_mode(pin2, pigpio.OUTPUT)
        self.pi.set_mode(pin3, pigpio.OUTPUT)
        self.pi.set_mode(pin4, pigpio.OUTPUT)
        self.pin1 = pin1
        self.pin2 = pin2
        self.pin3 = pin3
        self.pin4 = pin4
        self.delay_after_step = delay_after_step
        self.deque = deque(sequence)

    def do_counterclockwise_step(self):
        self.deque.rotate(-1)
        self.do_step_and_delay(self.deque[0])

    def do_clockwise_step(self):
        self.deque.rotate(1)
        self.do_step_and_delay(self.deque[0])

    def do_step_and_delay(self, step):
        self.pi.write(self.pin1, step[0])
        self.pi.write(self.pin2, step[1])
        self.pi.write(self.pin3, step[2])
        self.pi.write(self.pin4, step[3])
        sleep(self.delay_after_step)


# from stepit import StepperMotor





#!/usr/bin/env python

# servo_demo.py
# 2016-10-07
# Public Domain

# servo_demo.py          # Send servo pulses to GPIO 4.
# servo_demo.py 23 24 25 # Send servo pulses to GPIO 23, 24, 25.

import sys
import time
import random
import pigpio

NUM_GPIO = 32

MIN_WIDTH = 500
MAX_WIDTH = 2500

step = [0] * NUM_GPIO
width = [0] * NUM_GPIO
used = [False] * NUM_GPIO

pi = pigpio.pi()

if not pi.connected:
    exit()

G = [4]


for g in G:
    used[g] = True
    step[g] = random.randrange(50, 55)
    if step[g] % 2 == 0:
        step[g] = -step[g]
    width[g] = random.randrange(MIN_WIDTH, MAX_WIDTH + 1)

print("Sending servos pulses to GPIO {}, control C to stop.".
      format(' '.join(str(g) for g in G)))

while True:

    try:

        for g in G:

            pi.set_servo_pulsewidth(g, width[g])

            # print(g, width[g])

            width[g] += step[g]

            if width[g] < MIN_WIDTH or width[g] > MAX_WIDTH:
                step[g] = -step[g]
                width[g] += step[g]

        time.sleep(0.01)

    except KeyboardInterrupt:
        break

print("\nTidying up")

for g in G:
    pi.set_servo_pulsewidth(g, 0)

pi.stop()



"""
 Copyright (c) 2018-2019 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
import argparse
import json
import signal
import socket
import sys
import threading
import time

from python_banyan.gateway_base import GatewayBase


# noinspection PyAbstractClass
class Esp8266Gateway(GatewayBase, threading.Thread):
    """
    This class is a Python Banyan component that subscribes to topics requesting
    action or responses from an ESP8266. This component communicates with the
    ESP8266
    as a TCP client that connects to the ESP8266.

    It may also be configured as a TCP server, but then the code on the ESP8266
    will need to be updated with the IP address of the server.

    The main thread is the Banyan receive loop, and a separate thread is
    created to listen
    for TCP messages coming from the ESP826
    """

    def __init__(self, subscriber_list, connection_mode='client',
                 back_plane_ip_address=None,
                 subscriber_port='43125',
                 publisher_port='43124',
                 ip_address=None,
                 ip_port=31337,
                 ip_packet_length=96,
                 validate_pin=True,
                 wait_for_ip_address=False):
        """

        :param connection_mode: client (default) or server
        :param ip_address: ip address of the esp8266
        :param ip_port: ip port of the esp8266
        :param ip_packet_length: tcp packet length for communication
        with the esp8266
        :param validate_pin: validate the pin number
        """
        self.pin_info = {}
        self.gpio_pins = [4, 5, 12, 13, 14, 15]

        self.back_plane_ip_address = back_plane_ip_address
        self.publisher_port = publisher_port
        self.subscriber_port = subscriber_port

        self.subscriber_list = subscriber_list
        self.ip_port = ip_port

        # initialize the parents
        super(Esp8266Gateway, self).__init__(
            subscriber_list=self.subscriber_list,
            back_plane_ip_address=self.back_plane_ip_address,
            publisher_port=self.publisher_port,
            subscriber_port=self.subscriber_port,
            process_name='Esp8266Gateway')
        threading.Thread.__init__(self)
        self.daemon = True

        self.pkt_len = ip_packet_length
        self.validate_pin = validate_pin

        # in case we need to stop the thread
        self.stop_event = threading.Event()

        self.connection_socket = None

        # set up the socket and connect to the esp8266 server
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        if connection_mode == 'server':
            self.sock.bind(('', ip_port))
            self.sock.listen(1)
            print('Esp8266Gateway waiting for connection...')
            try:
                self.connection_socket, self.address = self.sock.accept()
            except KeyboardInterrupt:
                # self.clean_up()
                sys.exit(0)
            print("Connected to {}:{}".format(self.address[0], self.address[1]))
        else:
            # wait for ip address message before connecting
            if not wait_for_ip_address:
                # we are not waiting
                if not ip_address:
                    raise RuntimeError('For client, ip address must be specified')
                # addr_port = ip_address, ip_port
                self.sock.connect((ip_address, ip_port))
                self.connection_socket = self.sock
                print('Connected to server on remote device',
                      self.connection_socket)
                self.start()
        # start the thread to receive messages from the esp8266
        # self.start()

        # send system up message if other banyan components need a start trigger
        payload = {'sys_msg': 'system_up'}
        self.publish_payload(payload, 'sys_msg')

        # start the banyan receive loop
        try:
            self.receive_loop()
        except KeyboardInterrupt:
            self.clean_up()
            sys.exit(0)

    def init_pins_dictionary(self):
        """
        This method will initialize the pins dictionary contained
        in gateway base parent class. This method is called by
        the gateway base parent in its init method.

        NOTE: that this a a non-asyncio method
        :return:
        """
        # pins data structure
        # pins supported: GPIO4, GPIO5, GPIO12, GPIO13, GPIO14, GPIO15
        # modes - IN - PULL_UP, OUT - OPEN_DRAIN, PWM
        # irqs - Pin.IRQ_RISING | Pin.IRQ_FALLING

        # build a status table for the pins
        for x in self.gpio_pins:
            entry = {'mode': None, 'pull_up': False, 'drain': False,
                     'irq': None, 'duty': None, 'freq': None, 'count': 0,
                     'value': 0}
            self.pin_info[x] = entry

    def stop(self):
        """
        Called if the thread needs to be stopped
        :return:
        """
        self.stop_event.set()

    def is_stopped(self):
        """
        A test to determine if the thread is active or not
        :return:
        """
        return self.stop_event.is_set()

    def additional_banyan_messages(self, topic, payload):
        if payload['command'] == 'ip_address':
            if not self.connection_socket:
                try:
                    self.sock.connect((payload['address'], self.ip_port))
                except (ConnectionRefusedError, OSError, KeyError):
                    payload = {'report': 'panic'}
                    self.publish_payload(payload, 'from_esp8266_gateway')
                    raise
                self.connection_socket = self.sock
                print('Connected to server on remote device',
                      self.connection_socket)
                self.start()

    def digital_write(self, topic, payload):
        """
        This method performs a digital write
        :param topic: message topic
        :param payload: {"command": "digital_write", "pin": “PIN”, "value":
        “VALUE”}
        """
        self.send_payload_to_esp(payload)

    def disable_analog_reporting(self, topic, payload):
        """
        This method disables analog input reporting for the selected pin.
        :param topic: message topic
        :param payload: {"command": "disable_analog_reporting", "pin": “PIN”,
        "tag": "TAG"}
        """
        payload_to_esp = {'command': 'disable_analog_reporting', 'pin': 0}
        self.send_payload_to_esp(payload_to_esp)

    def disable_digital_reporting(self, topic, payload):
        """
        This method disables digital input reporting for the selected pin.

        :param topic: message topic
        :param payload: {"command": "disable_digital_reporting", "pin":
        “PIN”, "tag": "TAG"}
        """
        payload_to_esp = {'command': 'disable_digital_reporting',
                          'pin': payload['pin']}
        self.send_payload_to_esp(payload_to_esp)

    def i2c_read(self, topic, payload):
        """
        This method will perform an i2c read by specifying the i2c
        device address, i2c device register and the number of bytes
        to read.

        Call set_mode_i2c first to establish the pins for i2c operation.

        :param topic: message topic
        :param payload: {"command": "i2c_read", "pin": “PIN”, "tag": "TAG",
                         "addr": “I2C ADDRESS, "register": “I2C REGISTER”,
                         "number_of_bytes": “NUMBER OF BYTES”}
        :return via the i2c_callback method
        """
        self.send_payload_to_esp(payload)

    def i2c_write(self, topic, payload):
        """
        This method will perform an i2c write for the i2c device with
        the specified i2c device address, i2c register and a list of byte
        to write.

        Call set_mode_i2c first to establish the pins for i2c operation.

        :param topic: message topic
        :param payload: {"command": "i2c_write", "pin": “PIN”, "tag": "TAG",
                         "addr": “I2C ADDRESS, "register": “I2C REGISTER”,
                         "data": [“DATA IN LIST FORM”]}
        """
        self.send_payload_to_esp(payload)

    def play_tone(self, topic, payload):
        """
        This method plays a tone on a piezo device connected to the selected
        pin at the frequency and duration requested.
        Frequency is in hz and duration in milliseconds.

        Call set_mode_tone before using this method.
        :param topic: message topic
        :param payload: {"command": "play_tone", "pin": “PIN”, "tag": "TAG",
                         “freq”: ”FREQUENCY”, duration: “DURATION”}
        """
        self.send_payload_to_esp(payload)

    def pwm_write(self, topic, payload):
        """
        This method sets the pwm value for the selected pin.
        Call set_mode_pwm before calling this method.
        :param topic: message topic
        :param payload: {“command”: “pwm_write”, "pin": “PIN”,
                         "tag":”TAG”,
                          “value”: “VALUE”}
        """
        self.send_payload_to_esp(payload)

    def servo_position(self, topic, payload):
        """
        This method will set a servo's position in degrees.
        Call set_mode_servo first to activate the pin for
        servo operation.

        :param topic: message topic
        :param payload: {'command': 'servo_position',
                         "pin": “PIN”,'tag': 'servo',
                        “position”: “POSITION”}
        """
        self.send_payload_to_esp(payload)

    def set_mode_analog_input(self, topic, payload):
        """
        This method sets a GPIO pin as analog input.
        :param topic: message topic
        :param payload: {"command": "set_mode_analog_input", "pin": “PIN”,
        "tag":”TAG” }
        """
        payload = {'command': 'set_mode_analog_input'}
        self.send_payload_to_esp(payload)

    def set_mode_digital_input(self, topic, payload):
        """
        This method sets a pin as digital input.
        :param topic: message topic
        :param payload: {"command": "set_mode_digital_input", "pin": “PIN”,
        "tag":”TAG” }
        """
        # self, pin, pull_up=True, irq='both', count=0

        pin = payload['pin']
        payload_to_esp = {'command': 'set_mode_digital_input', 'pin': pin,
                          'pull_up': True, 'irq': 'both',
                          'count': 0}
        if self.is_valid_pin(pin):
            self.pin_info[pin]['mode'] = GatewayBase.DIGITAL_INPUT_MODE

            if 'pull_up' in payload:
                payload_to_esp['pull_up'] = payload['pull_up']
                self.pin_info[pin][
                    'mode'] = GatewayBase.DIGITAL_INPUT_PULLUP_MODE
            self.pin_info[pin]['pull_up'] = payload_to_esp['pull_up']

            if 'irq' in payload:
                payload_to_esp['irq'] = payload['irq']
            self.pin_info[pin]['irq'] = payload_to_esp['irq']

            if 'count' in payload:
                payload_to_esp['count'] = payload['count']
            self.pin_info[pin]['count'] = payload_to_esp['count']

            self.send_payload_to_esp(payload)
        else:
            # invalid pin number
            raise RuntimeError('set_mode_digital_input - invalid pin:', pin)

    # elif cmd == 'get_in':

    def set_mode_digital_input_pullup(self, topic, payload):
        """
        This method sets a pin as digital input with pull up enabled.
        :param topic: message topic
        :param payload: message payload
        """
        payload['pull_up'] = True
        self.set_mode_digital_input(topic, payload)

    def set_mode_digital_output(self, topic, payload):
        """
        This method sets a pin as a digital output pin.
        :param topic: message topic
        :param payload: {"command": "set_mode_digital_output", "pin": PIN,
        "tag":”TAG” }
        """
        # for the esp8266, we set the pin mode when fulfilling the action
        # for example - digital write
        pass

    def set_mode_i2c(self, topic, payload):
        """
        This method sets up the i2c pins for i2c operations.
        :param topic: message topic
        :param payload: {"command": "set_mode_i2c"}
        """
        self.send_payload_to_esp(payload)

    def set_mode_pwm(self, topic, payload):
        """
         This method sets a GPIO pin capable of PWM for PWM operation.
         :param topic: message topic
         :param payload: {"command": "set_mode_pwm", "pin": “PIN”, "tag":”TAG” }
         """
        # not used by the esp8266 gateway
        pass

    def set_mode_servo(self, topic, payload):
        """
        This method establishes a GPIO pin for servo operation.
        :param topic: message topic
        :param payload: {"command": "set_mode_servo", "pin": “PIN”,
        "tag":”TAG” }
        """
        pass

    def set_mode_sonar(self, topic, payload):
        """
        This method sets the trigger and echo pins for sonar operation.
        :param topic: message topic
        :param payload: {"command": "set_mode_sonar", "trigger_pin": “PIN”,
        "tag":”TAG”
                         "echo_pin": “PIN”"tag":”TAG” }
        """
        self.send_payload_to_esp(payload)

    def set_mode_stepper(self, topic, payload):
        """
        This method establishes either 2 or 4 GPIO pins to be used in stepper
        motor operation.
        :param topic:
        :param payload:{"command": "set_mode_stepper", "pins": [“PINS”],
                        "steps_per_revolution": “NUMBER OF STEPS”}
        """
        self.send_payload_to_esp(payload)

    def set_mode_tone(self, topic, payload):
        """
        Establish a GPIO pin for tone operation.
        :param topic:
        :param payload:{"command": "set_mode_tone", "pin": “PIN”, "tag":”TAG” }
        """
        pass

    def stepper_write(self, topic, payload):
        """
        Move a stepper motor for the specified number of steps.
        :param topic:
        :param payload: {"command": "stepper_write", "motor_speed": “SPEED”,
                         "number_of_steps":”NUMBER OF STEPS” }
        """
        self.send_payload_to_esp(payload)

    def send_payload_to_esp(self, payload):
        """
        Encode and send payload/topic to esp8266
        :param payload:
        """

        # run it through json and send it out
        payload = json.dumps(payload)

        payload = payload.ljust(self.pkt_len).encode('utf-8')

        # send the data out
        self.connection_socket.sendall(payload)

    def is_valid_pin(self, pin):
        """
        validate pin
        :param pin: pin number
        :return: True or False
        """
        if self.validate_pin:
            if pin in self.pin_info:
                return True
            else:
                return False
        else:
            return True

    def run(self):
        """
        This is thread that receives packets from the eps8266
        :return:
        """

        while True:
            try:
                payload = self.connection_socket.recv(self.pkt_len)

                pkt_len_received = len(payload)
                # if the packet size is less than the fixed
                # length packet size - Nagle's algorithm,
                # keep receiving data for a complete packet
                while pkt_len_received < self.pkt_len:
                    wait_for = self.pkt_len - pkt_len_received
                    short_packet = self.connection_socket.recv(wait_for)
                    payload += short_packet
                    pkt_len_received += len(payload)
            except OSError:
                # we turned non-blocking on, so this exception is expected
                continue
            try:
                # perform a json decode
                # the data is in the form of a dictionary
                payload = json.loads(payload)
                # if 'report' in payload:
                #     if payload['report'] == 'i2c_data':
                #         print(payload)

                payload['timestamp'] = time.time()
                self.publish_payload(payload, 'from_esp8266_gateway')

            except ValueError:
                print('EspWiFiBridge -  data: {} length {}'.format(
                    payload, len(payload)))
                raise ValueError


def esp8266_gateway():
    # allow user to bypass the IP address auto-discovery. This is necessary
    # if the component resides on a computer
    # other than the computing running the backplane.

    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="back_plane_ip_address", default="None",
                        help="None or IP address used by Back Plane")
    parser.add_argument("-c", dest="connection_mode", default="client",
                        help="client or server")
    # allow the user to specify a name for the component and have it shown on
    # the console banner.
    # modify the default process name to one you wish to see on the banner.
    # change the default in the derived class to set the name
    parser.add_argument("-i", dest="ip_address", default="None",
                        help="None or IP address of remote device if running "
                             "in client mode")
    parser.add_argument("-l", dest="ip_packet_length", default=96,
                        help="IP packet length - must match remote device")
    parser.add_argument("-m", dest="subscriber_list",
                        default=['to_esp8266_gateway'], nargs='+',
                        help="Banyan topics space delimited: topic1 topic2 "
                             "topic3")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")
    parser.add_argument("-t", dest="ip_port", default="31337",
                        help="ip connection port")
    parser.add_argument("-w", dest="wait_for_ip_address", default="False",
                        help="Wait ESP IP address message.")
    parser.add_argument("-x", dest="validate_pin", default="True",
                        help="Validate Pin Number - True or False")

    args = parser.parse_args()

    validate_pin = args.validate_pin.lower()
    if validate_pin == 'false':
        validate_pin = False
    else:
        validate_pin = True

    wait_for_ip_address = args.wait_for_ip_address.lower()
    if wait_for_ip_address == 'false':
        wait_for_ip_address = False
    else:
        wait_for_ip_address = True

    kw_options = {
        'connection_mode': args.connection_mode,
        'ip_address': args.ip_address,
        'ip_packet_length': args.ip_packet_length,
        'ip_port': int(args.ip_port),
        'validate_pin': validate_pin,
        'wait_for_ip_address': wait_for_ip_address
    }

    if args.back_plane_ip_address != 'None':
        kw_options['back_plane_ip_address'] = args.back_plane_ip_address

    Esp8266Gateway(args.subscriber_list, **kw_options)

    # Esp8266Gateway()


def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    # replace with name of function you defined above
    esp8266_gateway()



"""
 Copyright (c) 2019 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
import argparse
import psutil
import signal
import subprocess
import sys
import time
# import webbrowser


class S3A:
    """
    This class starts the Banyan server to support the Scratch 3 OneGPIO Arduino
    extension.

    It will start the backplane, arduino gateway and websocket gateway.
    """

    def __init__(self, com_port=None, arduino_instance_id=None, log=None):
        self.backplane_exists = False
        self.proc_bp = None
        self.proc_agw = None
        self.proc_hwg = None

        # checking running processes.
        # if the backplane is already running, just note that and move on.
        for pid in psutil.pids():
            p = psutil.Process(pid)
            if p.name() == "backplane":
                print("Backplane already running.          PID = " + str(p.pid))
                self.backplane_exists = True
            else:
                continue

        # if backplane is not already running, start a new instance
        if not self.backplane_exists:
            self.proc_bp = subprocess.Popen(['backplane'],
                                            stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                            stdout=subprocess.PIPE)
            # new_entry['process'] = self.proc
            # new_entry['process_id'] = self.proc.pid
            self.backplane_exists = True
            print('Backplane is now running')

        if sys.platform.startswith('win32'):
            wsgw_start = ['wsgw']
            ardgw_start = ['ardgw']
        else:
            wsgw_start = ['xterm', '-e', 'wsgw']
            ardgw_start = ['xterm', '-e', 'ardgw']

        if log:
            wsgw_start.append('-l')
            wsgw_start.append('True')
            ardgw_start.append('-l')
            ardgw_start.append('True')
        if com_port:
            ardgw_start.append('-c')
            ardgw_start.append(com_port)
        if arduino_instance_id:
            ardgw_start.append('-i')
            ardgw_start.append(arduino_instance_id)





        if sys.platform.startswith('win32'):
            self.proc_agw = subprocess.Popen(wsgw_start,
                                             creationflags=subprocess.CREATE_NEW_CONSOLE)
            self.proc_hwg = subprocess.Popen(ardgw_start,
                                             creationflags=subprocess.CREATE_NEW_CONSOLE)

        else:
            self.proc_awg = subprocess.Popen(wsgw_start,
                                                 stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                                 stdout=subprocess.PIPE)

            self.proc_hwg = subprocess.Popen(ardgw_start,
                                             stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                             stdout=subprocess.PIPE)

        # webbrowser.open('https://mryslab.github.io/s3onegpio/', new=1)

        while True:
            try:
                time.sleep(1)
            except KeyboardInterrupt:
                self.proc_awg.kill()
                self.proc_hwg.kill()
                sys.exit(0)

def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


def s3ax():
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", dest="com_port", default="None",
                        help="Use this COM port instead of auto discovery")
    parser.add_argument("-i", dest="arduino_instance_id", default="None",
                        help="Set an Arduino Instance ID and match it in FirmataExpress")
    parser.add_argument("-l", dest="log", default="False",
                        help="Set to True to turn logging on.")

    args = parser.parse_args()

    log = args.log.lower()
    if log == 'false':
        log = False
    else:
        log = True

    if args.com_port == "None":
        com_port = None
    else:
        com_port=args.com_port

    if args.arduino_instance_id == "None":
        arduino_instance_id = None
    else:
        arduino_instance_id = int(args.arduino_instance_id)

    if com_port and arduino_instance_id:
        raise RuntimeError('Both com_port arduino_instance_id were set. Only one is allowed')

    S3A(com_port=com_port, arduino_instance_id=args.arduino_instance_id, log=log)


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    # replace with name of function you defined above
    s3ax()



#!/usr/bin/env python3

"""
 This is the Python Banyan GUI that communicates with
 the Raspberry Pi Banyan Gateway

 Copyright (c) 2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
import argparse
# noinspection PyPackageRequirements
import logging
import pathlib
import serial
# noinspection PyPackageRequirements
from serial.tools import list_ports
import signal
import sys
import threading
import time
from python_banyan.banyan_base import BanyanBase


# noinspection PyMethodMayBeStatic
class PicoboardGateway(BanyanBase, threading.Thread):
    """
    This class is the interface class for the picoboard supporting
    Scratch 3.

    It will regularly poll the picoboard, normalize the sensor data, and then
    publish it. Data is published in the form of a list.
    """

    def __init__(self, back_plane_ip_address=None, subscriber_port='43125',
                 publisher_port='43124', process_name='PicoboardGateway',
                 com_port=None, publisher_topic=None, log=False):
        """
        :param back_plane_ip_address:
        :param subscriber_port:
        :param publisher_port:
        :param process_name:
        :param com_port: picoboard com_port
        """

        # initialize parent
        super(PicoboardGateway, self).__init__(back_plane_ip_address, subscriber_port,
                                               publisher_port, process_name=process_name)

        self.log = log
        if self.log:
            fn = str(pathlib.Path.home()) + "/pbgw.log"
            self.logger = logging.getLogger(__name__)
            logging.basicConfig(filename=fn, filemode='w', level=logging.DEBUG)
            sys.excepthook = self.my_handler

        self.baud_rate = 38400
        self.publisher_topic = publisher_topic

        # place to receive data from picoboard
        self.data_packet = None

        # data value positions in the data stream
        # generated by the picoboard

        # position 0 = board id
        # position 1 = D  analog inverted logic
        # position 2 = C  analog inverted logic
        # position 3 = B  analog inverted logic
        # position 4 = Button  digital inverted logic
        # position 5 = A  analog inverted logic
        # position 6 = Light  analog inverted logic
        # position 7 = sound  analog
        # position 8 = slider analog

        # positional indices into data stream for analog sensors
        # light is a special case and removed from this list
        self.analog_sensor_list = [1, 2, 3, 5, 7, 8]

        # positional values for specific sensor types
        self.button_position = 4
        self.light_position = 6
        self.sound_position = 7

        # indices that require data inversion.
        # light is left out of this list and handled separately.
        self.inverted_analog_list = [1, 2, 3, 5]

        # The payload data is built as a list of entries.
        # Indices are as follows:
        # index 0 = D  analog inverted logic
        # index 1 = C  analog inverted logic
        # index 2 = B  analog inverted logic
        # index 3 = Button  digital inverted logic
        # index 4 = A  analog inverted logic
        # index 5 = Light  analog inverted logic
        # index 6 = sound  analog
        # index 7 = slider analog

        self.payload = {'report': []}

        # poll request for picoboard data
        self.poll_byte = b'\x01'

        # if a com port was specified use it.
        if com_port:
            self.picoboard = serial.Serial(com_port, self.baud_rate,
                                           timeout=1, writeTimeout=0)
        # otherwise try to find a picoboard
        else:
            if self.find_the_picoboard():
                print('picoboard found on:', self.picoboard.port)
            else:
                print('Cannot find Picoboard')
                sys.exit(0)
            # self.picoboard.reset_input_buffer()
            # self.picoboard.reset_output_buffer()

        # start the thread to receive data from the picoboard
        threading.Thread.__init__(self)
        self.daemon = True
        self.stop_event = threading.Event()
        self.start()

        # allow thread time to start
        time.sleep(.2)

        while True:
            try:
                time.sleep(.001)
                # request data
                self.picoboard.write(self.poll_byte)
            except (KeyboardInterrupt, serial.SerialException):
                self.picoboard.reset_input_buffer()
                self.picoboard.reset_output_buffer()
                self.picoboard.close()
                sys.exit(0)

    def find_the_picoboard(self):
        """
        Go through the ports looking for an active board
        """

        the_ports_list = list_ports.comports()
        for port in the_ports_list:
            if port.pid is None:
                continue
            else:
                print('Looking for picoboard on: ', port.device)
                self.picoboard = serial.Serial(port.device, self.baud_rate,
                                               timeout=1, writeTimeout=0)

                self.picoboard.close()
                self.picoboard.open()
                # send multiple polls to allow for serial port to open
                for send in range(10):
                    self.picoboard.write(self.poll_byte)
                    time.sleep(.2)
                    num_bytes = self.picoboard.inWaiting()
                    if num_bytes == 18:
                        self.picoboard.reset_input_buffer()
                        self.picoboard.reset_output_buffer()
                        return True
                    else:
                        continue
            continue
        return False

    def analog_scaling(self, value, index):
        """
        scale the normal analog input range of 0-1023 to 0-100
        :param value:
        :param index: sensor index value within data stream
        :return: A value scaled between 0 and 100
        """
        if index == self.light_position:
            input_low = 0
            input_high = 100
        elif index in self.inverted_analog_list:  # the light channel
            input_low = 1023
            input_high = 0
        else:
            input_low = 0
            input_high = 1023

        new_value_low = 0
        new_value_high = 100

        return round(((value - input_low) * ((new_value_high - new_value_low) / (input_high - input_low))) +
                     new_value_low)

    def my_handler(self, xtype, value, tb):
        """
        for logging uncaught exceptions
        :param xtype:
        :param value:
        :param tb:
        :return:
        """
        self.logger.exception("Uncaught exception: {0}".format(str(value)))

    def run(self):
        """
        This method run continually, receiving responses
        to the poll request.

        Formulas for adjusting light and sound are from this URL:
        https://twiki.cern.ch/twiki/pub/Sandbox/DaqSchoolExercise14/A_pico.py.txt
        """
        while True:
            self.data_packet = None
            # if there is data available from the picoboard
            # retrieve 18 bytes - a full picoboard packet
            cooked = None
            if self.picoboard.inWaiting():
                self.data_packet = self.picoboard.read(18)
                # get the channel number and data for the channel
                for i in range(9):
                    # pico_channel = self.channels[(int(self.data_packet[2 * i]) - 128) >> 3]
                    raw_sensor_value = ((int(self.data_packet[2 * i]) & 7) << 7) + int(self.data_packet[2 * i + 1])
                    if i == 0:  # id
                        cooked = raw_sensor_value
                    elif i == self.light_position:
                        if raw_sensor_value < 25:
                            cooked = 100 - raw_sensor_value
                        else:
                            cooked = round((1023-raw_sensor_value) * (75/998))
                        cooked = self.analog_scaling(cooked, self.light_position)

                    elif i == self.sound_position:
                        n = max(0, raw_sensor_value - 18)
                        if n < 50:
                            cooked = int(n / 2)
                        else:
                            cooked = 25 + min(75, int((n - 50) * (75 / 580)))
                    elif i == self.button_position:  # invert digital input
                        cooked = int(not raw_sensor_value)

                    if i in self.analog_sensor_list:
                        # scale for standard analog:
                        cooked = self.analog_scaling(raw_sensor_value, i)

                    # don't add the firmware id to the payload -
                    # the extension does not need it.
                    if i != 0:
                        self.payload['report'].append(cooked)

                self.publish_payload(self.payload, self.publisher_topic)
                # print(self.payload)
                self.payload = {'report': []}
            else:
                # no data available, just kill some time
                try:
                    time.sleep(.001)
                except KeyboardInterrupt:
                    sys.exit(0)


def picoboard_gateway():
    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="back_plane_ip_address", default="None",
                        help="None or IP address used by Back Plane")
    parser.add_argument("-c", dest="com_port", default="None",
                        help="Use this COM port instead of auto discovery")
    parser.add_argument("-l", dest="log", default="False",
                        help="Set to True to turn logging on.")
    parser.add_argument("-n", dest="process_name",
                        default="PicoboardGateway", help="Set process name in "
                                                         "banner")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-r", dest="publisher_topic",
                        default="from_picoboard_gateway", help="Report topic")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")

    args = parser.parse_args()

    kw_options = {
        'publisher_port': args.publisher_port,
        'subscriber_port': args.subscriber_port,
        'process_name': args.process_name,
        'publisher_topic': args.publisher_topic
    }

    if args.back_plane_ip_address != 'None':
        kw_options['back_plane_ip_address'] = args.back_plane_ip_address

    if args.com_port != 'None':
        kw_options['com_port'] = args.com_port

    log = args.log.lower()
    if log == 'false':
        log = False
    else:
        log = True

    kw_options['log'] = log

    PicoboardGateway(**kw_options)


# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    picoboard_gateway()



#!/usr/bin/env python3

"""
 This is the Python Banyan GUI that communicates with
 the Raspberry Pi Banyan Gateway

 Copyright (c) 2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
import argparse
# noinspection PyPackageRequirements
import logging
import pathlib
import serial
# noinspection PyPackageRequirements
from serial.tools import list_ports
import signal
import sys
import threading
import time
from python_banyan.banyan_base import BanyanBase


# noinspection PyMethodMayBeStatic
class PicoboardGateway(BanyanBase, threading.Thread):

    """
    This class is the interface class for the picoboard supporting
    Scratch 3.

    It will regularly poll the picoboard, normalize the sensor data, and then
    publish it.
    """

    def __init__(self, back_plane_ip_address=None, subscriber_port='43125',
                 publisher_port='43124', process_name='PicoboardGateway',
                 com_port=None, publisher_topic=None, log=False):
        """
        :param back_plane_ip_address:
        :param subscriber_port:
        :param publisher_port:
        :param process_name:
        :param com_port: picoboard com_port
        """

        super(PicoboardGateway, self).__init__(back_plane_ip_address, subscriber_port,
                                               publisher_port, process_name=process_name)

        self.log = log
        if self.log:
            fn = str(pathlib.Path.home()) + "/pbgw.log"
            self.logger = logging.getLogger(__name__)
            logging.basicConfig(filename=fn, filemode='w', level=logging.DEBUG)
            sys.excepthook = self.my_handler

        self.baud_rate = 38400
        self.publisher_topic = publisher_topic

        # place to receive data from picoboard
        self.data_packet = None

        # data value positions in the data stream
        # generated by the picoboard

        # data value 0 = board id
        # data value 1 = D  analog inverted logic
        # data value 2 = C  analog inverted logic
        # data value 3 = B  analog inverted logic
        # data value 4 = Button  digital inverted logic
        # data value 5 = A  analog inverted logic
        # data value 6 = Light  analog inverted logic
        # data value 7 = sound  analog
        # data value 8 = slider analog

        # indices into data stream for analog sensors
        self.analog_sensor_list = [1, 2, 3, 5, 6, 7, 8]

        self.button_index = 4

        self.light_index = 6

        self.sound_index = 7

        # indices that require data inversion
        self.inverted_analog_list = [1, 2, 3, 5]

        # payload used to publish picoboard values
        # The keys map to the data value positions
        # listed above.
        # self.payload = {'report': {0: 0, 1: 0, 2: 0, 3: 0,
        #                 4: 0, 5: 0,
        #                 6: 0, 7: 0, 8: 0}}

        self.payload = {'report': []}

        # poll request for picoboard data
        self.poll_byte = b'\x01'

        # if a com port was specified use it.
        if com_port:
            self.picoboard = serial.Serial(com_port, self.baud_rate,
                                           timeout=1, writeTimeout=0)
        # otherwise try to find a picoboard
        else:
            if self.find_the_picoboard():
                print('picoboard found on:', self.picoboard.port)
            else:
                print('Cannot find Picoboard')
            self.picoboard.reset_input_buffer()
            self.picoboard.reset_output_buffer()

        # start the thread to receive data from the picoboard
        threading.Thread.__init__(self)
        self.daemon = True
        self.stop_event = threading.Event()
        self.start()

        # allow thread time to start
        time.sleep(.2)

        while True:
            try:
                time.sleep(.001)
                # request data
                self.picoboard.write(self.poll_byte)
            except KeyboardInterrupt:
                self.picoboard.reset_input_buffer()
                self.picoboard.reset_output_buffer()
                self.picoboard.close()
                sys.exit(0)

    def find_the_picoboard(self):
        """
        Go through the ports looking for an active board
        """
        #
        the_ports_list = list_ports.comports()
        for port in the_ports_list:
            if port.pid is None:
                continue
            else:
                print('Looking for picoboard on: ', port.device)
                self.picoboard = serial.Serial(port.device, self.baud_rate,
                                               timeout=1, writeTimeout=0)

                self.picoboard.close()
                self.picoboard.open()
                # send multiple polls to allow for serial port to open
                for send in range(10):
                    self.picoboard.write(self.poll_byte)
                    time.sleep(.2)
                    num_bytes = self.picoboard.inWaiting()
                    if num_bytes == 18:
                        self.picoboard.reset_input_buffer()
                        self.picoboard.reset_output_buffer()
                        return True
                    else:
                        continue
            continue
        return False

    def analog_scaling(self, value, index):
        """
        scale the normal analog input range of 0-1023 to 0-100
        :param value:
        :param index: sensor index value within data stream
        :return: A value scaled between 0 and 100
        """
        if index in self.inverted_analog_list:  # the light channel
            input_low = 1023
            input_high = 0
        else:
            input_low = 0
            input_high = 1023

        new_value_low = 0
        new_value_high = 100

        return round(((value - input_low) * ((new_value_high - new_value_low) / (input_high - input_low))) +
                     new_value_low)

    def my_handler(self, xtype, value, tb):
        """
        for logging uncaught exceptions
        :param xtype:
        :param value:
        :param tb:
        :return:
        """
        self.logger.exception("Uncaught exception: {0}".format(str(value)))

    def run(self):
        """
        This method run continually, receiving responses
        to the poll request.

        Formulas for adjusting light and sound are from this URL:
        https://twiki.cern.ch/twiki/pub/Sandbox/DaqSchoolExercise14/A_pico.py.txt
        """
        while True:
            self.data_packet = None
            # if there is data available from the picoboard
            # retrieve 18 bytes - a full picoboard packet
            cooked = None
            if self.picoboard.inWaiting():
                self.data_packet = self.picoboard.read(18)
                # get the channel number and data for the channel
                for i in range(9):
                    # pico_channel = self.channels[(int(self.data_packet[2 * i]) - 128) >> 3]
                    raw_sensor_value = ((int(self.data_packet[2 * i]) & 7) << 7) + int(self.data_packet[2 * i + 1])
                    if i == 0:  # id
                        cooked = raw_sensor_value
                    elif i == self.light_index:
                        if raw_sensor_value < 25:
                            cooked = 100 - raw_sensor_value
                        else:
                            cooked = round((1023-raw_sensor_value) * (75/998))
                    elif i == self.sound_index:
                        n = max(0, raw_sensor_value - 18)
                        if n < 50:
                            cooked = int(n / 2)
                        else:
                            cooked = 25 + min(75, int((n - 50) * (75 / 580)))
                    elif i == self.button_index:  # invert digital input
                        cooked = int(not raw_sensor_value)

                    if i in self.analog_sensor_list:
                        # scale for standard analog:
                        cooked = self.analog_scaling(raw_sensor_value, i)

                    self.payload['report'].append(cooked)

                self.publish_payload(self.payload, self.publisher_topic)
                # print(self.payload)
            else:
                # no data available, just kill some time
                try:
                    time.sleep(.001)
                except KeyboardInterrupt:
                    sys.exit(0)


def picoboard_gateway():
    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="back_plane_ip_address", default="None",
                        help="None or IP address used by Back Plane")
    parser.add_argument("-c", dest="com_port", default="None",
                        help="Use this COM port instead of auto discovery")
    parser.add_argument("-l", dest="log", default="False",
                        help="Set to True to turn logging on.")
    parser.add_argument("-n", dest="process_name",
                        default="PicoboardGateway", help="Set process name in "
                                                         "banner")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-r", dest="publisher_topic",
                        default="from_picoboard", help="Report topic")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")

    args = parser.parse_args()

    kw_options = {
        'publisher_port': args.publisher_port,
        'subscriber_port': args.subscriber_port,
        'process_name': args.process_name,
        'publisher_topic': args.publisher_topic
    }

    if args.back_plane_ip_address != 'None':
        kw_options['back_plane_ip_address'] = args.back_plane_ip_address

    if args.com_port != 'None':
        kw_options['com_port'] = args.com_port

    log = args.log.lower()
    if log == 'false':
        log = False
    else:
        log = True

    kw_options['log'] = log

    PicoboardGateway(**kw_options)


# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    picoboard_gateway()
